﻿/*
 * Copyright (C) 2020 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include "ukws_close_button.h"
#include <QDebug>
#include <QStyle>
#include <QStyleOption>
#include <QApplication>

UkwsCloseButton::UkwsCloseButton(QWidget *parent): QLabel(parent)
{
    btnSize = QSize(BTN_DEF_WIDTH, BTN_DEF_HEIGHT);
    resize(btnSize);

    default_closePix = QIcon(":/image/ukui-taskview-close-default.svg").pixmap(btnSize.width(), btnSize.width());
    hover_closePix = QIcon(":/image/ukui-taskview-close-hover.svg").pixmap(btnSize.width(), btnSize.width());
    click_closePix = QIcon(":/image/ukui-taskview-close-click.svg").pixmap(btnSize.width(), btnSize.width());

    m_btnState = BTN_STATE_NORMAL;

    m_pCircleTimer = new QTimer(this);
    m_pCircleTimer->setInterval(50);
    connect(m_pCircleTimer, &QTimer::timeout, this, &UkwsCloseButton::onTimeout);
}

void UkwsCloseButton::setBtnSize(int width, int height)
{
    resize(QSize(width, height));
}

void UkwsCloseButton::setBtnState(BottonState state)
{
    if (state >= BTN_STATE_NUM)
    {
        qWarning("the close button state %d is invalid.", state);
        return;
    }

    if (state != m_btnState)
    {
        m_btnState = state;
        switch(m_btnState){
            case BTN_STATE_NORMAL:
                btnStateNormal();
                break;
            case BTN_STATE_ENTER:
                btnStateEnter();
                break;
            case BTN_STATE_PRESS:
                btnStatePress();
                break;
            case BTN_STATE_HOVER:
                btnStateHover();
                break;
            default:
                btnStateUndef();
                break;
        }
    }

    return;
}

void UkwsCloseButton::btnStateNormal()
{
    //qDebug()<<"enter btnStateNormal==============";
    QLabel::clear();
    setStyleSheet("background: rgba(0, 0, 0, 0)");
    return;
}

void UkwsCloseButton::btnStateEnter()
{
    //qDebug()<<"enter btnStateEnter==============";
    setPixmap(hover_closePix);
    raise();
    return;
}

void UkwsCloseButton::btnStatePress()
{
    //qDebug()<<"enter btnStatePress==============";
    setPixmap(click_closePix);
    emit buttonClicked();
    return;
}

void UkwsCloseButton::btnStateHover()
{
    //qDebug()<<"enter btnStateHover==============";
    setPixmap(default_closePix);

//    this->setPixmap(QIcon(":/image/ukui-taskview-close-default.png").pixmap(btnSize.width(), btnSize.width()));
//    setStyleSheet("background: rgba(0, 0, 0, 0)");
    return;
}

void UkwsCloseButton::btnStateUndef()
{
    qDebug("close button state %d undefined",m_btnState);
    return;
}

void UkwsCloseButton::paintEvent(QPaintEvent *event)
{
    if (BTN_STATE_ENTER == m_btnState)
    {
//        setStyleSheet("background: rgba(0, 0, 0, 0)");

        QPainter painter(this);
        painter.setRenderHint(QPainter::Antialiasing); // 反锯齿;
//        painter.setBrush(QBrush(Qt::red));
        painter.setPen(Qt::transparent);
        QRect rect = this->rect();
        rect.setWidth(rect.width());
        rect.setHeight(rect.height());
//        painter.drawRoundedRect(rect, 3, 3);

        painter.translate(btnSize.width() / 2, btnSize.height() / 2); // 设置旋转中心
        painter.rotate(m_nAngle); // 旋转
        painter.translate(-(btnSize.width() / 2), -(btnSize.height() / 2)); // 将原点复位

        painter.drawPixmap(rect, hover_closePix);
    } else {
        QLabel::paintEvent(event);
    }
}

void UkwsCloseButton::startCircle()
{
    m_pCircleTimer->start();
}

void UkwsCloseButton::onTimeout()
{
    ++m_nCirleCount;
    if(m_nAngle == 90) // 旋转 90度
    {
        m_nAngle = 0;
    }
    //m_nAngle += 15 ;

    if(m_nCirleCount == 6)
    {
        m_pCircleTimer->stop();
        m_nCirleCount = 0;
    }
    update();
}
