﻿/*
 * Copyright (C) 2020 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include "ukws_common.h"
#include "ukws_workspace_box.h"
#include "ukws_workspace_manager.h"

#include <QDebug>
#include <QScrollArea>
#include <QtX11Extras/QX11Info>
#include <QScreen>
#include <QApplication>
#include <QScreen>
#include <QStyleOption>
#include <QPainter>
#include <QX11Info>
#include <QString>
#include <iostream>
#include <ukuisdk/kylin-com4c.h>
#include <ukuisdk/kylin-com4cxx.h>
#include <QSettings>

#include "ukws_stack_blur.h"

// ukws_helper.h包含X.h，必须放到所有Qt头文件之前
#include "ukws_helper.h"

// 打开多任务视图时 禁止旋转 退出时恢复
#define SETTINGS_TABLET_MODE    "org.ukui.SettingsDaemon.plugins.tablet-mode"
#define AUTO_ROTATION           "auto-rotation"

// 打开多任务视图时 获取当前主题
#define UKUI_STYLE    "org.ukui.style"
#define STYLE_NAME    "style-name"

UkwsWorkspaceManager::UkwsWorkspaceManager(QWidget *parent) : QWidget(parent)
{

    wmOperator = new UkwsWnckOperator;
    indStack = new QStackedWidget;
    newWorkspaceBtn = new UkwsNewWorkspaceButton;  

    kylin_statusmanger_interface = new QDBusInterface(
        "com.kylin.statusmanager.interface", "/",
        "com.kylin.statusmanager.interface", QDBusConnection::sessionBus());



    QString layside = KDKGetPrjCodeName().c_str();
    if (layside == "V10SP1-edu") {
        layDesktopSideBar = false;
    } else {
        layDesktopSideBar = true;
    }

    QDBusConnection::sessionBus().connect(
                "com.kylin.statusmanager.interface", "/",
                "com.kylin.statusmanager.interface", "mode_change_signal",
        this, SLOT(tabletchanged(bool)));


    qDebug()<<"layDesktopSideBar ="<<layside;

    if (layDesktopSideBar) {
        mainLayout = new QVBoxLayout;
        wsboxLayout = new QHBoxLayout;
        //topSpacer = new QSpacerItem(0, 0, QSizePolicy::Fixed);

        resetTimer = new QTimer;
        showWMCompleted = false;

        showStatus = UkwsWidgetShowStatus::Hidden;
        config = nullptr;

        mainLayout->addWidget(indStack);
        mainLayout->addLayout(wsboxLayout);
        //wsboxLayout->addSpacerItem(topSpacer);

        mainLayout->setAlignment(Qt::AlignHCenter);
        wsboxLayout->setAlignment(Qt::AlignTop | Qt::AlignHCenter);

        mainLayout->setMargin(0);
        wsboxLayout->setMargin(0);
    } else {
        mainLayout = new QVBoxLayout;
        //topSpacer = new QSpacerItem(0, 0, QSizePolicy::Fixed);
        resetTimer = new QTimer;
        showWMCompleted = false;

        showStatus = UkwsWidgetShowStatus::Hidden;
        config = nullptr;

        mainLayout->addWidget(indStack);

        mainLayout->setAlignment(Qt::AlignHCenter);

        mainLayout->setMargin(0);
    }


    this->setLayout(mainLayout);
    this->resize(1200, 600);

    this->installEventFilter(this);

    connect(resetTimer, &QTimer::timeout, this, [this](){
        showWMCompleted = false;// ? showWMCompleted = false : showWMCompleted = true;
        resetTimer->stop();
    });

    QByteArray id(UKUI_STYLE);
    if (QGSettings::isSchemaInstalled(id)){
        style_Setting = new QGSettings(id);
    }

}

void UkwsWorkspaceManager::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event);
    QStyleOption styleOpt;
    styleOpt.init(this);
    QPainter painter(this);
    style()->drawPrimitive(QStyle::PE_Widget, &styleOpt, &painter, this);

    QWidget::paintEvent(event);
}

void UkwsWorkspaceManager::reloadWorkspace(int minScale)
{
    cleanAllWorkspace();

    // 获取主屏区域信息
    QList<QScreen *> screenList = QGuiApplication::screens();
    QScreen* primaryScreen = QGuiApplication::primaryScreen();
    QRect screenRect = primaryScreen->geometry();
    int screenNum = screenList.indexOf(primaryScreen);
    WnckScreen *screen = wnck_screen_get(screenNum);

    float scale = (float)config->workspaceItemUnits / config->workspaceAllUnits;

    int w = screenRect.width() * scale;
    int h = screenRect.height() * scale;

    this->resize(screenRect.width(), screenRect.height());

    wmOperator->updateWorkspaceList();
    int size = wmOperator->workspaceQList->size();


    //topSpacer->changeSize(w, screenRect.height() / 40);

    QPixmap wsboxBackground = background.scaled(QSize(w, h),
                                                Qt::IgnoreAspectRatio,
                                                Qt::SmoothTransformation);


    for (int i = 0; i < size; i++) {
        WnckWorkspace *wws = wmOperator->workspaceQList->at(i);
        UkwsWorkspaceBox *wsbox = new UkwsWorkspaceBox;
        UkwsIndicator *ind = new UkwsIndicator;

        wsbox->index = i;
        QLocale local;
        QString language=local.languageToString(local.language());
        QString name;
        if (QString::compare(language,"Chinese")==0)
        {
           name = "工作区"+QString::number(i+1);
        } else {
           name = "Workspace"+QString::number(i+1);
        }



        wsbox->setTitle(name);
        wsbox->setSizeByThumbnailSize(w, h); 
        wsbox->setWnckWorkspace(wws);
        wsbox->setBackground(wsboxBackground);

        if (layDesktopSideBar) {
            ind->setFixedHeight(screenRect.height() * config->workspacePrimaryAreaUnits
                               / config->workspaceAllUnits);
        } else {
            ind->setFixedWidth(screenRect.width() * config->workspacePrimaryAreaUnits
                               / config->workspaceAllUnits);
        }

        ind->setConfig(config);
        ind->wmOperator->screen = screen;
        ind->wmOperator->workspace = wws;
        ind->wmOperator->needCheckWorkspace = true;
        ind->wmOperator->needCheckScreen = false;
        ind->index = i;
//        ind->reShow(UkwsIndicator::ShowModeTiling, minScale);
        ind->setAcceptDrops(true);
        ind->setObjectName(UKWS_OBJ_IND_MAINWIDGET_Tiling);

        connect(wsbox, &UkwsWorkspaceBox::doHover,
                this, &UkwsWorkspaceManager::setShowingIndicator);
        connect(wsbox, &UkwsWorkspaceBox::selectedWorkspace,
                this, &UkwsWorkspaceManager::changeWorkspace);
        connect(wsbox, &UkwsWorkspaceBox::windowChangeWorkspace,
                this, &UkwsWorkspaceManager::moveWindowWorkspace);
        connect(wsbox, &UkwsWorkspaceBox::windowdelete,
                this, &UkwsWorkspaceManager::deleteWindowWorkspace);
        connect(ind, &UkwsIndicator::isSelected,
                this, &UkwsWorkspaceManager::selectWinbox);
        connect(ind, &UkwsIndicator::windowViewPixmapChange,
                this, &UkwsWorkspaceManager::doIndicatorWindowViewChange);
        connect(ind, &UkwsIndicator::closeWindowRefresh,
                this, &UkwsWorkspaceManager::doIndicatorWindowViewChange);
        connect(ind, &UkwsIndicator::needBeHide,
                this, &UkwsWorkspaceManager::closeWorkspace);

        spaceBoxList.append(wsbox);
        indList.append(ind);

        if (layDesktopSideBar) {
            wsboxLayout->addWidget(wsbox);
        }

        indStack->addWidget(ind);
    }


    for (int i = 0; i < size; i++) {

        indList.at(i)->reShow(UkwsIndicator::ShowModeTiling,!layDesktopSideBar,minScale);

        if (!indList.at(i)->windowListcheckTimer.isActive())
            indList.at(i)->windowListcheckTimer.start();
    }


    // 显示当前的工作区
    WnckWorkspace *workspace = wnck_screen_get_active_workspace(screen);
    int workspaceNum = wnck_workspace_get_number(workspace);
    setShowingIndicator(workspaceNum);

    if (layDesktopSideBar) {
        wsboxLayout->addWidget(newWorkspaceBtn);
        newWorkspaceBtn->show();
    } else {
        newWorkspaceBtn->hide();
    }

    newWorkspaceBtn->setSizeByButtonSize(w, h);
   // newWorkspaceBtn->show();
    //newWorkspaceBtn->hide();


    if (size >= UKWS_WORKSPACE_MAX)
        newWorkspaceBtn->hide();


    //  wnck_screen_change_workspace_count(wmOperator->screen,1);
    if (!newspace) {
        newspace = true;
        connect(newWorkspaceBtn, &UkwsNewWorkspaceButton::addWorkspace,this, [=](int wbIndex,int srcWsIndex,int dstWsIndex){

            //WnckWorkspace *wws = wmOperator->workspaceQList->at(workspacesize);

            wmOperator->updateWorkspaceList();
            int size = wmOperator->workspaceQList->size();
            wnck_screen_change_workspace_count(wmOperator->screen,size+1);

            QTimer::singleShot(100,this,[=](){
                wmOperator->updateWorkspaceList();
                WnckWorkspace *wws = wmOperator->workspaceQList->at(size);

                UkwsWorkspaceBox *wsbox = new UkwsWorkspaceBox;
                UkwsIndicator *ind = new UkwsIndicator;

                wsbox->index = size;

                QLocale local;
                QString language=local.languageToString(local.language());
                QString name;
                if (QString::compare(language,"Chinese")==0)
                {
                   name = "工作区"+QString::number(size+1);
                } else {

                   name = "Workspace"+QString::number(size+1);
                }

                wsbox->setTitle(name);
                wsbox-> setSizeByThumbnailSize(w, h);
                wsbox->setWnckWorkspace(wws);
                wsbox->setBackground(wsboxBackground);

                ind->setFixedHeight(screenRect.height() * config->workspacePrimaryAreaUnits
                                   / config->workspaceAllUnits);
                ind->setConfig(config);
                ind->wmOperator->screen = screen;
                ind->wmOperator->workspace = wws;
                ind->wmOperator->needCheckWorkspace = true;
                ind->wmOperator->needCheckScreen = false;
                ind->index = size;
        //        ind->reShow(UkwsIndicator::ShowModeTiling, minScale);
                ind->setAcceptDrops(true);
                ind->setObjectName(UKWS_OBJ_IND_MAINWIDGET_Tiling);

                connect(wsbox, &UkwsWorkspaceBox::doHover,
                        this, &UkwsWorkspaceManager::setShowingIndicator);
                connect(wsbox, &UkwsWorkspaceBox::selectedWorkspace,
                        this, &UkwsWorkspaceManager::changeWorkspace);
                connect(wsbox, &UkwsWorkspaceBox::windowChangeWorkspace,
                        this, &UkwsWorkspaceManager::moveWindowWorkspace);
                connect(wsbox, &UkwsWorkspaceBox::windowdelete,
                        this, &UkwsWorkspaceManager::deleteWindowWorkspace);
                connect(ind, &UkwsIndicator::isSelected,
                        this, &UkwsWorkspaceManager::selectWinbox);
                connect(ind, &UkwsIndicator::windowViewPixmapChange,
                        this, &UkwsWorkspaceManager::doIndicatorWindowViewChange);
                connect(ind, &UkwsIndicator::closeWindowRefresh,
                        this, &UkwsWorkspaceManager::doIndicatorWindowViewChange);
                connect(ind, &UkwsIndicator::needBeHide,
                        this, &UkwsWorkspaceManager::closeWorkspace);

                spaceBoxList.append(wsbox);
                indList.append(ind);

                if (layDesktopSideBar) {
                    wsboxLayout->insertWidget(size,wsbox);
                }

                indStack->addWidget(ind);


               indList.at(size)->reShow(UkwsIndicator::ShowModeTiling,!layDesktopSideBar,minScale);

               if (!indList.at(size)->windowListcheckTimer.isActive())
                   indList.at(size)->windowListcheckTimer.start();




                if ((size+1) >= UKWS_WORKSPACE_MAX)
                    newWorkspaceBtn->hide();
                if (dstWsIndex == -1) {
                    moveWindowWorkspace(wbIndex,srcWsIndex,dstWsIndex);
                }
            });




        });
    }


}

void UkwsWorkspaceManager::reShow(int minScale)
{
    if(showWMCompleted == true)
        return;
    if (showStatus != UkwsWidgetShowStatus::Hidden) {
        return;
    }


    QDBusMessage response =kylin_statusmanger_interface->call("get_auto_rotation");
    autoRation = response.arguments().takeFirst().toBool();
    // 旋转的开关 开启就关闭,随后恢复；关闭就保持,不处理
    if (autoRation) {
        kylin_statusmanger_interface->call("set_auto_rotation",false,"ukui-window-switch","close rotation");
    }

    showStatus = UkwsWidgetShowStatus::Constructing;

    // 先让界面显示出来，从而让Qt完成布局重绘
    QScreen *primaryScreen = QGuiApplication::primaryScreen();
    this->move(primaryScreen->geometry().topLeft());
    QRect screenRect = primaryScreen->geometry();
    this->resize(screenRect.width(), screenRect.height());
    qDebug()<<"screenRect.height()"<<screenRect.height();

    // 获取背景图片
    this->getBackground();

    // 设置工作区视图的最底层背景
    this->setBackgroundImage(screenRect.width(), screenRect.height());

    // 背景设置完毕，留出时间处理绘制事件
    QCoreApplication::processEvents(QEventLoop::AllEvents, 5);

    // 全屏并提前显示背景
    this->showFullScreen();
    QCoreApplication::processEvents(QEventLoop::AllEvents, 200);

    reloadWorkspace(minScale);

    this->activateWindow();

    showStatus = UkwsWidgetShowStatus::Shown;

    showWMCompleted = true;
    resetTimer->start(1500);
}
void UkwsWorkspaceManager::tabletchanged (bool tablet)
{
    if (!tablet) {
        reHide();
    }
}
void UkwsWorkspaceManager::reHide()
{
    if(showWMCompleted == true)
        return;
    if (showStatus != UkwsWidgetShowStatus::Shown) {
        return;
    }

    if (autoRation) {
        kylin_statusmanger_interface->call("set_auto_rotation",true,"ukui-window-switch","opent rotation");
    }

    showStatus = UkwsWidgetShowStatus::Destructing;

    this->hide();
    // 优先处理hide事件，完成后再清理后续
    QCoreApplication::processEvents();

    cleanAllWorkspace();

    showStatus = UkwsWidgetShowStatus::Hidden;
    emit isHidden();

    //resetTimer->start(1000);
}

void UkwsWorkspaceManager::setShowingIndicator(int index)
{
    int size = indList.size();
    int idx = index;

    if (size <= 0) {
        qCritical("Indicator list is empty, but get index:%d ...\n", index);
        return;
    }

    // wnck在部分窗管上会出现各种异常问题，所以需要在此做边界判断
    if ((idx < 0) || (idx >= indList.size())) {
        qCritical("Illegal index values: %d, set index to 0\n", index);
        idx = 0;
    }

    for (int i = 0; i < size; i++)
        if (i != idx)
            indList.at(i)->hide();
        else
            indList.at(i)->show();

    indStack->setCurrentIndex(idx);
    UkwsIndicator *ind = indList.at(idx);
    ind->flowReLayout(layDesktopSideBar);

    UkwsWorkspaceBox *wsbox;
    foreach (wsbox, spaceBoxList) {
        wsbox->setSelectStatus(false);
    }
    wsbox = spaceBoxList.at(idx);
    wsbox->setSelectStatus(true);
}

void UkwsWorkspaceManager::selectWinbox(bool needActivate)
{
    if (needActivate) {
        UkwsIndicator *ind = static_cast<UkwsIndicator *>(indStack->currentWidget());
        ind->acitveSelectedWindow();
        reHide();
    }
}

void UkwsWorkspaceManager::changeWorkspace(int index)
{
    WnckWorkspace *workspace = wnck_screen_get_workspace(wmOperator->screen, index);
    unsigned long timestamp = QX11Info::getTimestamp();
    wnck_workspace_activate(workspace, timestamp);
    reHide();
}

void UkwsWorkspaceManager::deleteWindowWorkspace(int wbIndex)
{
    UkwsIndicator *srcInd = indList.at(wbIndex);

    UkwsIndicator *dstInd = indList.at(0);
    wmOperator->updateWorkspaceList();

    if (wmOperator->workspaceQList->size() <= 1) {
        return;
    }

    if (wbIndex == 0) {
        int size = indList.size();
        wbIndex++;
        for (int i = wbIndex; i < size; i++) {

            UkwsIndicator *srcInd2 = indList.at(i);

            UkwsIndicator *dstInd2 = indList.at(i-1);
            if (srcInd2->winboxList.size() > 0) {
            foreach(UkwsWindowBox *wb, srcInd2->winboxList) {
                wb->moveToWorkspace(i-1);

            //    wb->setWinboxSizeByHeight(boxMinHeight);

                // 从源indicator中移除
                srcInd2->rmWinbox(wb);

                // 根据dst的布局，更新winbox的大小

                // 添加到目标indicator中
                dstInd2->addWinbox(wb);

                // 更新桌面视图

            }

            srcInd2->updateWindowViewPixmap(true);
            dstInd2->updateWindowViewPixmap(true);

            dstInd2->reShow();
            } else {
                continue;
            }
        }

    } else {

        if (srcInd->winboxList.size() > 0) {

            foreach(UkwsWindowBox *wb, srcInd->winboxList) {
                wb->moveToWorkspace(0);

            //    wb->setWinboxSizeByHeight(boxMinHeight);

                // 从源indicator中移除
                srcInd->rmWinbox(wb);

                // 根据dst的布局，更新winbox的大小

                // 添加到目标indicator中
                dstInd->addWinbox(wb);

                // 更新桌面视图

            }
            srcInd->updateWindowViewPixmap(true);
            dstInd->updateWindowViewPixmap(true);

            dstInd->reShow();
        }
        int size = indList.size();
        wbIndex++;
        for (int i = wbIndex; i < size; i++) {

            UkwsIndicator *srcInd2 = indList.at(i);

            UkwsIndicator *dstInd2 = indList.at(i-1);

            if (srcInd2->winboxList.size() > 0) {
            foreach(UkwsWindowBox *wb, srcInd2->winboxList) {

                wb->moveToWorkspace(i-1);

            //    wb->setWinboxSizeByHeight(boxMinHeight);

                // 从源indicator中移除
                srcInd2->rmWinbox(wb);

                // 根据dst的布局，更新winbox的大小

                // 添加到目标indicator中
                dstInd2->addWinbox(wb);

                // 更新桌面视图

            }

            srcInd2->updateWindowViewPixmap(true);
            dstInd2->updateWindowViewPixmap(true);

            dstInd2->reShow();
            } else {

                continue;
            }
        }
    }


    wmOperator->updateWorkspaceList();
    int size2 = wmOperator->workspaceQList->size();
    wnck_screen_change_workspace_count(wmOperator->screen,size2-1);




    UkwsWorkspaceBox *wsbox = spaceBoxList.at(size2-1);

    wsboxLayout->removeWidget(wsbox);
    spaceBoxList.removeOne(wsbox);
    wsbox->deleteLater();


    UkwsIndicator *ind = indList.at(size2-1);
    ind->windowListcheckTimer.stop();
    indStack->removeWidget(ind);
    indList.removeOne(ind);
    ind->cleanAllWinbox();
    ind->deleteLater();





    if ((size2-1) < UKWS_WORKSPACE_MAX)
        newWorkspaceBtn->show();

    setShowingIndicator(0);

    // 重新布局

}


void UkwsWorkspaceManager::moveWindowWorkspace(int wbIndex, int srcWsIndex, int dstWsIndex)
{
    if (dstWsIndex == -1) {
        wmOperator->updateWorkspaceList();
        int size = wmOperator->workspaceQList->size();
        dstWsIndex = size - 1;
    }

    if (srcWsIndex >= indList.size() || (dstWsIndex >= indList.size()))
        return;
    qDebug()<<"wbIndex"<<wbIndex;
    qDebug()<<"srcWsIndex"<<srcWsIndex;
    qDebug()<<"dstWsIndex"<<dstWsIndex;

    UkwsIndicator *srcInd = indList.at(srcWsIndex);
    UkwsIndicator *dstInd = indList.at(dstWsIndex);
    UkwsWindowBox *wb = srcInd->getWinbox(wbIndex);

    if (wb == nullptr)
        return;

    wb->moveToWorkspace(dstWsIndex);

//    wb->setWinboxSizeByHeight(boxMinHeight);

    // 从源indicator中移除
    srcInd->rmWinbox(wb);

    // 根据dst的布局，更新winbox的大小

    // 添加到目标indicator中
    dstInd->addWinbox(wb);

    // 更新桌面视图
    srcInd->updateWindowViewPixmap(true);
    dstInd->updateWindowViewPixmap(true);

    dstInd->reShow();
    

    // 重新布局
//    srcInd->flowReLayout();
//    dstInd->flowReLayout();
}

void UkwsWorkspaceManager::doIndicatorWindowViewChange(int indIndex)
{
    if (indIndex >= indList.size())
        return;

    if (indIndex >= spaceBoxList.size())
        return;

    UkwsWorkspaceBox *wsbox = spaceBoxList.at(indIndex);
    UkwsIndicator *wsind = indList.at(indIndex);
    wsbox->updateDesktopViewThumbnail(indList.at(indIndex)->getWindowView());


}

QString UkwsWorkspaceManager::getBackgroundFileByGSettings(QString schemaDir,
                                                           QString schemaUrl,
                                                           QString keyName)
{
    QUrl url;
    QString fileUrl = QString("");

    /*
     * 不再使用glib的方式获取gsettings，使用QGsettings
     */
#if 0
    GSettingsSchemaSource *schema_source = NULL;
    GSettingsSchema *schema = NULL;
    GSettings *settings = NULL;
    GVariant *value = NULL;
    const gchar *name;

    schema_source = g_settings_schema_source_new_from_directory (schemaDir.toStdString().c_str(),
                                                                 g_settings_schema_source_get_default(),
                                                                 TRUE, NULL);
    if (!schema_source) {
        qWarning() << "Cannot get gsettings schema form" << schemaDir;
        goto GSFREE;
    }

    schema = g_settings_schema_source_lookup(schema_source,
                                             schemaUrl.toStdString().c_str(),
                                             FALSE);
    if (!schema) {
        qWarning() << "Cannot get gsettings schema source:" << schemaUrl;
        goto GSFREE;
    }

    settings = g_settings_new_full(schema, NULL, NULL);
    if (!settings) {
        qWarning() << "Cannot get gsettings from schema";
        goto GSFREE;
    }

    value = g_settings_get_value(settings, keyName.toStdString().c_str());
    if (!value)  {
        qWarning() << "Cannot get key:" << keyName;
        goto GSFREE;
    }

    name = g_variant_get_string(value, NULL);
    if (!name)  {
        qWarning() << QString("Cannot get gsettings key(%1) value").arg(keyName);
        goto GSFREE;
    }

    url = QUrl::fromEncoded(QByteArray(name));
    fileUrl = url.toString().replace(QRegExp("^file:/"), "");

GSFREE:
    if (value != NULL)
        g_variant_unref(value);

    if (settings != NULL)
        g_object_unref(settings);

    if (schema != NULL)
        g_settings_schema_unref(schema);

    if (schema_source != NULL)
        g_settings_schema_source_unref(schema_source);
#endif

    Q_UNUSED(schemaDir);

    if (QGSettings::isSchemaInstalled(schemaUrl.toUtf8())) {
        QGSettings gsettings(schemaUrl.toUtf8());

        const QStringList keyList = gsettings.keys();
//        qDebug() << schemaUrl << "Key List:" << keyList;
        if (keyList.contains(keyName)) {
            QString name = gsettings.get(keyName).toString();
            url = QUrl::fromLocalFile(name);
            fileUrl = url.toString().replace(QRegExp("^file:/"), "");
//            qDebug() << "Get background file:" << fileUrl;
        } else {
            qWarning() << QString("Cannot get gsettings key(%1) value").arg(keyName);
        }
    }

    return fileUrl;
}

QString UkwsWorkspaceManager::getBackgroundColorGSettings(QString schemaDir,
                                                          QString schemaUrl,
                                                          QString keyName)
{
    QString colorString = "";

    /*
     * 不再使用glib的方式获取gsettings，使用QGsettings
     */

    Q_UNUSED(schemaDir);

    if (QGSettings::isSchemaInstalled(schemaUrl.toUtf8())) {
        QGSettings gsettings(schemaUrl.toUtf8());

        const QStringList keyList = gsettings.keys();
        if (keyList.contains(keyName)) {
            colorString = gsettings.get(keyName).toString();
//            qDebug() << "Get background color:" << colorString;
        } else {
            qWarning() << QString("Cannot get gsettings key(%1) value").arg(keyName);
        }
    }

    return colorString;
}

void UkwsWorkspaceManager::getBackground()
{
    QString filePath;

    filePath = getBackgroundFileByGSettings("/usr/share/glib-2.0/schemas/",
                                            "org.mate.background",
                                            "pictureFilename");

    if (filePath == "") {
        // 无法从ukui桌面环境获取背景文件，则获取背景颜色
        QString colorName = getBackgroundColorGSettings("/usr/share/glib-2.0/schemas/",
                                                        "org.mate.background",
                                                        "primaryColor");
        if (colorName != "") {
            if (background.isNull()) {
                background = QPixmap(480, 270);
            }
            background.fill(QColor(colorName));
            return;
        }

        // 无法获取背景色，则继续获取gnome桌面环境的背景图片
        filePath = getBackgroundFileByGSettings("/usr/share/glib-2.0/schemas/",
                                                "org.gnome.desktop.background",
                                                "pictureUri");
        if (filePath == "") {
            qWarning() << "Cannot get background image, use default:" << UKWS_WORKSPACE_DEFAULT_BACKGROUND;
            filePath = UKWS_WORKSPACE_DEFAULT_BACKGROUND;
        }
    }

    QImage img;
    img.load(filePath);
    background = QPixmap::fromImage(img);
}

void UkwsWorkspaceManager::setBackgroundImage(int width, int height)
{
    // 计算主区域宽度
    int w, h, x;

    if (width == 0)
        w = size().width();
    else
        w = width;



    if (height == 0)
        h = size().height();
    else
        h = height;

    x = h * config->workspacePrimaryAreaUnits / config->workspaceAllUnits;


    // 背景图置灰
    QPixmap tempPixmap = background.scaled(QSize(w, h), Qt::IgnoreAspectRatio,
                                           Qt::SmoothTransformation);
    UkwsStackBlur blurrer;
    blurrer.setOrigImage(tempPixmap.toImage());
    blurrer.setReduce(2);
    blurrer.setRadius(16);
    //blurrer.setBlurRect(QRect(x, 0, w - x + 1, size().height()));
    blurrer.setBlurRect(QRect(0, 0, w, size().height()));
    blurrer.calculate();
    tempPixmap = QPixmap::fromImage(blurrer.blurImage());

    QPainter painter;
    painter.begin(&tempPixmap);
    painter.setOpacity(0.25);
    // 设置上半区域遮罩
    painter.fillRect(0, 0, w, x, QColor(19, 19, 20));

    painter.setOpacity(0.75);
    // 设置下半区域遮罩

    if (style_Setting->get(STYLE_NAME).toString() == "ukui-dark") {
        painter.fillRect(0, x, w, h-x, QColor(35, 36, 38));
        painter.end();
    } else {
        painter.fillRect(0, x, w, h-x, QColor(245, 245, 245));
        painter.end();
    }

    setAutoFillBackground(true);   // 这个属性一定要设置
    QPalette pal(palette());
    pal.setBrush(QPalette::Window,QBrush(tempPixmap));
    setPalette(pal);
}

void UkwsWorkspaceManager::cleanAllWorkspace()
{
    // 清理workspace box

    if (layDesktopSideBar) {
        foreach(UkwsWorkspaceBox *wsbox, spaceBoxList) {
            wsboxLayout->removeWidget(wsbox);
            spaceBoxList.removeOne(wsbox);
            wsbox->deleteLater();
        }
        spaceBoxList.clear();
        wsboxLayout->removeWidget(newWorkspaceBtn);
    }


    // 清理indicator
    foreach(UkwsIndicator *ind, indList) {
        ind->windowListcheckTimer.stop();
        indStack->removeWidget(ind);
        indList.removeOne(ind);
        ind->cleanAllWinbox();
        ind->deleteLater();
    }
    indList.clear();
}

void UkwsWorkspaceManager::setConfig(UkwsConfig *config)
{
    this->config = config;
}

void UkwsWorkspaceManager::closeWorkspace()
{
    reHide();
}

bool UkwsWorkspaceManager::eventFilter(QObject *object, QEvent *event)
{


    if (object == this) {
        if (event->type() == QEvent::MouseButtonPress) {
            return true;
        }

        if (event->type() == QEvent::MouseButtonRelease){
            //changeWorkspace(indStack->currentIndex());
            //reHide();
            if (!layDesktopSideBar) {
                //视图下 mouse点击-滑动-松开不关闭
                if (doscroll == 0) {
                    reHide();
                } else {
                    doscroll = 0;
                }
            }
            return true;
        }

        if (event->type() == QEvent::MouseMove) {
            if (!layDesktopSideBar) {
                doscroll = 1; //视图下 mouse点击-滑动-松开不关闭
            }
            return true;
        }

        // QEvent::KeyPress == 6
        if (event->type() == 6) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
            if (keyEvent->key() == Qt::Key_Escape) {
                reHide();
                return true;
            }
        }

        if (event->type() == QEvent::WindowDeactivate) {
            if (showStatus == UkwsWidgetShowStatus::Shown ||
                     showStatus == UkwsWidgetShowStatus::Constructing) {

                changeWorkspace(indStack->currentIndex());

                reHide();
            }
            return true;
        }
    }

    return false;
}
